@@ -21,18 +21,27 @@
 
 #pragma once
 
-#if defined(__GNUC__) && !defined(CAPNP_HEADER_WARNINGS)
-#pragma GCC system_header
-#endif
-
 #if CAPNP_LITE
 #error "Reflection APIs, including this header, are not available in lite mode."
 #endif
 
+#undef CONST
+// For some ridiculous reason, Windows defines CONST to const. We have an enum value called CONST
+// in schema.capnp.h, so if this is defined, compilation is gonna fail. So we undef it because
+// that seems strictly better than failing entirely. But this could cause trouble for people later
+// on if they, say, include windows.h, then include schema.h, then include another windows API
+// header that uses CONST. I suppose they may have to re-#define CONST in between, or change the
+// header ordering. Sorry.
+//
+// Please don't file a bug report telling us to change our enum naming style. You are at least
+// seven years too late.
+
 #include <capnp/schema.capnp.h>
 #include <kj/hash.h>
 #include <kj/windows-sanity.h>  // work-around macro conflict with `VOID`
 
+CAPNP_BEGIN_HEADER
+
 namespace capnp {
 
 class Schema;
@@ -117,6 +126,10 @@ public:
   BrandArgumentList getBrandArgumentsAtScope(uint64_t scopeId) const;
   // Gets the values bound to the brand parameters at the given scope.
 
+  kj::Array<uint64_t> getGenericScopeIds() const;
+  // Returns the type IDs of all parent scopes that have generic parameters, to which this type is
+  // subject.
+
   StructSchema asStruct() const;
   EnumSchema asEnum() const;
   InterfaceSchema asInterface() const;
@@ -143,6 +156,9 @@ public:
   kj::StringPtr getShortDisplayName() const;
   // Get the short version of the node's display name.
 
+  const kj::StringPtr getUnqualifiedName() const;
+  // Get the display name "nickname" of this node minus the prefix
+
 private:
   const _::RawBrandedSchema* raw;
 
@@ -260,6 +276,28 @@ public:
   // there is no such field.  (If the schema does not represent a union or a struct containing
   // an unnamed union, then this always returns null.)
 
+  bool isStreamResult() const;
+  // Convenience method to check if this is the result type of a streaming RPC method.
+
+  bool mayContainCapabilities() const { return raw->generic->mayContainCapabilities; }
+  // Returns true if a struct of this type may transitively contain any capabilities. I.e., are
+  // any of the fields an interface type, or a struct type that may in turn contain capabilities?
+  //
+  // This is meant for optimizations where various bookkeeping can possibly be skipped if it is
+  // known in advance that there are no capabilities. Note that this may conservatively return true
+  // spuriously, e.g. if it would be inconvenient to compute the correct answer. A false positive
+  // should never cause incorrect behavior, just potentially hurt performance.
+  //
+  // It's important to keep in mind that even if a schema has no capability-typed fields today,
+  // they could always be added in future versions of the schema. So, just because the schema
+  // doesn't contain capabilities does NOT necessarily mean that an instance of the struct can't
+  // contain capabilities. However, it is a pretty good hint that the application won't plan to
+  // use such capabilities -- for example, if there are no caps in an RPC call's response type
+  // according to the client's version of the schema, then the client clearly isn't going to try
+  // to make any pipelined calls. The server could be operating with a new version of the schema
+  // and could actually return capabilities, but for the client to make a pipelined call, the
+  // client would have to know in advance that capabilities could be returned.
+
 private:
   StructSchema(Schema base): Schema(base) {}
   template <typename T> static inline StructSchema fromImpl() {
@@ -491,6 +529,9 @@ public:
   inline uint16_t getOrdinal() const { return ordinal; }
   inline uint getIndex() const { return ordinal; }
 
+  bool isStreaming() const { return getResultType().isStreamResult(); }
+  // Check if this is a streaming method.
+
   StructSchema getParamType() const;
   StructSchema getResultType() const;
   // Get the parameter and result types, including substituting generic parameters.
@@ -974,3 +1015,5 @@ inline Type Type::wrapInList(uint depth) const {
 }
 
 }  // namespace capnp
+
+CAPNP_END_HEADER
